#
# Create a code field to base the georelational link
# This was originally coded as a numeric field, but
# it must be recoded to a label field in order to
# be linked back to the GIS database using a relational
# join.
#
$db allocate georef CODE
$db calculate {georef=format("%-.15g",VALUE)}

#
# Convert the numeric species codes into categories.
# This will change fields SP1, SP2, etc... into sp1, sp2, etc.
# Use a loop to do this for all species components.
#
for {set i 1} {$i < 6} {incr i} {
    $db allocate sp$i CODE
    $db calculate "sp$i=format(\"%-.15g\",SP$i)"
    $db delete SP$i
}

#
# Add in the coefficients for the volume equations.
# These are linked in by the 'sp1' field.
# Use the output command to tell the spreadsheet not to
# bother displaying these coefficients.
#
$db join load coef coef.rul
$db output off t1 t2 v1 v2 v3 b1 b2 q1 q2 q3 a1 a2 h1 h2 h3

#
# Define the current date and age of each stand
#
$db define date {1994}
$db define age {date-ORIGIN}

#
# Calculate the site index for each forest record
#
# Create a field to contain the site index value, and then apply the
# equations for each species type.
#
$db allocate si50 NUMBER

$db select set
$db select and {code=="pl"}
$db calculate {si50=14.2007+0.7087*HT-0.6659*(log(age)^2)+6.0486*(HT/age)-0.00285*age*log(age)}

$db select set
$db select and {code=="sw"}
$db calculate {si50=21.7888+0.5564*HT-0.9144*(log(HT)^2)-4.1848*log(age)-(45.2407/age) + 24.2757*(HT/age)}

$db select set
$db select and {code=="sb"}
$db calculate {si50=9.9531+0.4751*HT-0.5524*log(HT)-0.4288*log(age)^2-(21.7297/age)+24.2757*(HT/age)}

$db select set
$db select and {code=="aw"}
$db calculate {si50=34.037-1.8141*log(age)^2-(80.5660/age)+0.2248*HT*log(HT)+0.0121*age*log(age)}

$db select set


#
# Now build in the height, volume and size relationships
# Since these are specified as production rules (using the
# define command) the values will automatically update
# as soon as the value of 'date' (the current simulation
# date) changes.
#
# Note that these expression are all defined as conditionals.
# If the value of coefficient 'a1' is 0 (meaning that the proper
# coefficients are not available), then the expression will evalute
# to 0.  Otherwise, the expression will be calculated accordingly.
#
$db define bhage {a1?(==,0,0):(age-(a1+a2/si50))}
$db define ht {a1?(==,0,0):1.3+(si50-1.3)*(1+exp(h1+h2*log(50)+h3*log(si50-1.3)))/(1+exp(h1+h2*log(bhage)+h3*log(si50-1.3)))}
$db define topht {a1?(==,0,0):1.3+t1*(ht-1.3)^t2}
$db define volume {a1?(==,0,0):v1*(1-exp(v3*(topht-1.3)))/(1+v2*exp(v3*(topht-1.3)))}
$db define ba {a1?(==,0,0):-1+b1*(volume+1)^b2}
$db define qdia {a1?(==,0,0):(-q1*log(1-(topht-1.3)/q2))^q3}
$db define ntrees {a1?(==,0,0):ba/(0.00007854*qdia^2)}

#
# Define some formats to prevent excessive precision
#
$db format si50 "%.1f"
$db format bhage "%.0f"
$db format ht "%.1f"
$db format topht "%.1f"
$db  format volume "%.1f"
$db format ba "%.1f"
$db  format qdia "%.2f"
$db  format ntrees "%.0f"

#
# That is all it takes to set up the model.  The model
# can be advanced by changing the value of 'date'.  Type the
# following command into the script window:
#  set [set db](date) 2005
# to advance the simulation time.
#
